@@ -1,6 +1,6 @@
 /* Lisp eval
 
-   Copyright (c) 2001, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (c) 2001, 2005-2011 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -54,9 +54,8 @@ static fentry fentry_table[] = {
 static fentry *
 get_fentry (const char *name)
 {
-  size_t i;
   assert (name);
-  for (i = 0; i < fentry_table_size; ++i)
+  for (size_t i = 0; i < fentry_table_size; ++i)
     if (STREQ (name, fentry_table[i].name))
       return &fentry_table[i];
   return NULL;
@@ -87,8 +86,7 @@ get_function_doc (const char *name)
 const char *
 get_function_name (Function p)
 {
-  size_t i;
-  for (i = 0; i < fentry_table_size; ++i)
+  for (size_t i = 0; i < fentry_table_size; ++i)
     if (fentry_table[i].func == p)
       return fentry_table[i].name;
   return NULL;
@@ -101,8 +99,8 @@ size_t
 countNodes (le * branch)
 {
   int count;
-
-  for (count = 0; branch; branch = branch->next, count++);
+  for (count = 0; branch; branch = branch->next, count++)
+    ;
   return count;
 }
 
@@ -121,15 +119,11 @@ evaluateBranch (le * trybranch)
     keyword = leNew (trybranch->data);
 
   if (keyword->data == NULL)
-    {
-      leWipe (keyword);
-      return leNIL;
-    }
+    return leNIL;
 
   func = get_fentry (keyword->data);
-  leWipe (keyword);
   if (func)
-    return func->func (1, false, trybranch);
+    return call_command (func->func, 1, false, trybranch);
 
   return NULL;
 }
@@ -167,16 +161,13 @@ The symbols sym are variables; they are literal (not evaluated).
 The values val are expressions; they are evaluated.
 +*/
 {
-  le *newvalue = leNIL, *current;
-  size_t argc = countNodes (arglist);
+  le *newvalue = leNIL;
 
-  if (arglist != NULL && argc >= 2)
+  if (arglist != NULL && countNodes (arglist) >= 2)
     {
-      for (current = arglist->next; current;
+      for (le *current = arglist->next; current;
            current = current->next->next)
         {
-          if (newvalue != leNIL)
-            leWipe (newvalue);
           newvalue = evaluateNode (current->next);
           set_variable (current->data, newvalue->data);
           if (current->next == NULL)
@@ -192,48 +183,42 @@ void
 leEval (le * list)
 {
   for (; list; list = list->next)
-    leWipe (evaluateBranch (list->branch));
+    evaluateBranch (list->branch);
 }
 
 le *
 execute_with_uniarg (bool undo, int uniarg, bool (*forward) (void), bool (*backward) (void))
 {
-  int uni, ret = true;
-  bool (*func) (void) = forward;
-
   if (backward && uniarg < 0)
     {
-      func = backward;
+      forward = backward;
       uniarg = -uniarg;
     }
   if (undo)
-    undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
-  for (uni = 0; ret && uni < uniarg; ++uni)
-    ret = func ();
+    undo_start_sequence ();
+  bool ret = true;
+  for (int uni = 0; ret && uni < uniarg; ++uni)
+    ret = forward ();
   if (undo)
-    undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
+    undo_end_sequence ();
 
   return bool_to_lisp (ret);
 }
 
 le *
-execute_function (const char *name, int uniarg)
+move_with_uniarg (int uniarg, bool (*move) (int dir))
+{
+  bool ret = true;
+  for (unsigned long uni = 0; ret && uni < (unsigned) abs (uniarg); ++uni)
+    ret = move (uniarg < 0 ? - 1 : 1);
+  return bool_to_lisp (ret);
+}
+
+le *
+execute_function (const char *name, int uniarg, bool is_uniarg)
 {
   Function func = get_function (name);
-  Macro *mp;
-
-  if (func)
-    return func (uniarg, true, NULL);
-  else
-    {
-      mp = get_macro (name);
-      if (mp)
-        {
-          call_macro (mp);
-          return leT;
-        }
-      return leNIL;
-    }
+  return func ? call_command (func, uniarg, is_uniarg, NULL) : NULL;
 }
 
 DEFUN ("execute-extended-command", execute_extended_command)
@@ -241,25 +226,22 @@ DEFUN ("execute-extended-command", execute_extended_command)
 Read function name, then read its arguments and call it.
 +*/
 {
-  char *name;
   astr msg = astr_new ();
 
   if (lastflag & FLAG_SET_UNIARG)
     {
       if (lastflag & FLAG_UNIARG_EMPTY)
-        astr_afmt (msg, "C-u ");
+        msg = astr_fmt ("C-u ");
       else
-        astr_afmt (msg, "%d ", uniarg);
+        msg = astr_fmt ("%d ", uniarg);
     }
   astr_cat_cstr (msg, "M-x ");
 
-  name = minibuf_read_function_name (astr_cstr (msg));
-  astr_delete (msg);
+  castr name = minibuf_read_function_name (astr_cstr (msg));
   if (name == NULL)
     return false;
 
-  ok = execute_function (name, uniarg);
-  free (name);
+  ok = bool_to_lisp (execute_function (astr_cstr (name), uniarg, lastflag & FLAG_SET_UNIARG) == leT);
 }
 END_DEFUN
 
@@ -267,27 +249,23 @@ END_DEFUN
  * Read a function name from the minibuffer.
  */
 static History *functions_history = NULL;
-char *
+castr
 minibuf_read_function_name (const char *fmt, ...)
 {
   va_list ap;
-  char *ms;
   Completion *cp = completion_new (false);
-  size_t i;
 
-  for (i = 0; i < fentry_table_size; ++i)
+  for (size_t i = 0; i < fentry_table_size; ++i)
     if (fentry_table[i].interactive)
       gl_sortedlist_add (get_completion_completions (cp), completion_strcmp,
                          xstrdup (fentry_table[i].name));
-  add_macros_to_list (get_completion_completions (cp), completion_strcmp);
 
   va_start (ap, fmt);
-  ms = minibuf_vread_completion (fmt, "", cp, functions_history,
-                                 "No function name given",
-                                 minibuf_test_in_completions,
-                                 "Undefined function name `%s'", ap);
+  castr ms = minibuf_vread_completion (fmt, "", cp, functions_history,
+                                       "No function name given",
+                                       minibuf_test_in_completions,
+                                       "Undefined function name `%s'", ap);
   va_end (ap);
-  free_completion (cp);
 
   return ms;
 }
