@@ -34,7 +34,7 @@
  * Convert a key chord into its ASCII representation
  */
 astr
-chordtostr (size_t key)
+chordtodesc (size_t key)
 {
   astr as = astr_new ();
 
@@ -294,16 +294,16 @@ keystrtovec (const char *key)
 }
 
 /*
- * Convert a key code sequence into a key code sequence string.
+ * Convert a key code sequence into a descriptive string.
  */
 astr
-keyvectostr (gl_list_t keys)
+keyvectodesc (gl_list_t keys)
 {
   astr as = astr_new ();
 
   for (size_t i = 0; i < gl_list_size (keys); i++)
     {
-      astr key = chordtostr ((size_t) gl_list_get_at (keys, i));
+      astr key = chordtodesc ((size_t) gl_list_get_at (keys, i));
       astr_cat (as, key);
       if (i < gl_list_size (keys) - 1)
         astr_cat_char (as, ' ');
