@@ -36,7 +36,14 @@
 #include "gl_xlist.h"
 #include "unused-parameter.h"
 
+#ifdef HAVE_GC_H
+#include <gc.h>
+#else
+#include <gc/gc.h>
+#endif
+
 #include "astr.h"
+#include "estr.h"
 #include "lists.h"
 
 #define ZILE_VERSION_STRING	"GNU " PACKAGE_NAME " " VERSION
@@ -46,8 +53,6 @@
  *--------------------------------------------------------------------------*/
 
 /* Opaque types. */
-typedef struct Line Line;
-typedef struct Region Region;
 typedef struct Marker Marker;
 typedef struct History History;
 typedef struct Undo Undo;
@@ -58,21 +63,12 @@ typedef struct Window Window;
 typedef struct Completion Completion;
 
 /* FIXME: Types which should really be opaque. */
-typedef struct Point Point;
+typedef struct Region Region;
 
-struct Point
+struct Region
 {
-  Line *p;			/* Line pointer. */
-  size_t n;			/* Line number. */
-  size_t o;			/* Offset. */
-};
-
-/* Undo delta types. */
-enum
-{
-  UNDO_REPLACE_BLOCK,		/* Replace a block of characters. */
-  UNDO_START_SEQUENCE,		/* Start a multi operation sequence. */
-  UNDO_END_SEQUENCE		/* End a multi operation sequence. */
+  size_t start;		/* The region start. */
+  size_t end;		/* The region end. */
 };
 
 enum
@@ -110,7 +106,6 @@ enum
   void                                                          \
   set_ ## name ## _ ## field (Obj *p, const char *field)        \
   {                                                             \
-    free (p->field);                                            \
     p->field = field ? xstrdup (field) : NULL;                  \
   }
 
@@ -147,19 +142,15 @@ typedef le * (*Function) (long uniarg, bool is_uniarg, le * list);
   DEFUN_ARGS(zile_func, c_func, args)
 
 /* String argument. */
-#define STR_ARG(name)                   \
-  char *name = NULL;                    \
-  bool free_ ## name = true;
-#define STR_INIT(name)                  \
-  if (arglist && arglist->next)         \
-    {                                   \
-      name = arglist->next->data;       \
-      arglist = arglist->next;          \
-      free_ ## name = false;            \
+#define STR_ARG(name)                           \
+  castr name = NULL;
+#define STR_INIT(name)                                  \
+  if (arglist && arglist->next)                         \
+    {                                                   \
+      if (arglist->next->data)                          \
+        name = astr_new_cstr (arglist->next->data);     \
+      arglist = arglist->next;                          \
     }
-#define STR_FREE(name)                  \
-  if (free_ ## name)                    \
-    free (name);
 
 /* Integer argument. */
 #define INT_ARG(name) \
@@ -213,11 +204,12 @@ typedef le * (*Function) (long uniarg, bool is_uniarg, le * list);
  * Keyboard handling.
  *--------------------------------------------------------------------------*/
 
-#define GETKEY_DELAYED                  0001
-#define GETKEY_UNFILTERED               0002
+/* Standard pauses in ds */
+#define GETKEY_DEFAULT                  -1
+#define GETKEY_DELAYED                  2000
 
 /* Special value returned for invalid key codes, or when no key is pressed. */
-#define KBD_NOKEY                       UINT_MAX
+#define KBD_NOKEY                       SIZE_MAX
 
 /* Key modifiers. */
 #define KBD_CTRL                        01000
@@ -267,13 +259,11 @@ typedef le * (*Function) (long uniarg, bool is_uniarg, le * list);
 /* Zile font codes */
 #define FONT_NORMAL		0000
 #define FONT_REVERSE		0001
+#define FONT_UNDERLINE		0002
 
 /* Custom exit code */
 #define EXIT_CRASH	2
 
-/* Default waitkey pause in ds */
-#define WAITKEY_DEFAULT 20
-
 
 /*--------------------------------------------------------------------------
  * Things that should really be in gnulib.
@@ -281,13 +271,5 @@ typedef le * (*Function) (long uniarg, bool is_uniarg, le * list);
 
 /* String comparison */
 #define STREQ(a, b) (strcmp (a, b) == 0)
-#define STRNEQ(a, b) (strcmp (a, b) != 0)
-
-/* Avoid warnings about functions that don't return. */
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 8)
-# define _GL_ATTRIBUTE_NORETURN __attribute__ ((__noreturn__))
-#else
-# define _GL_ATTRIBUTE_NORETURN /* empty */
-#endif
 
 #endif /* !ZILE_H */
