@@ -30,19 +30,17 @@
 #include <config.h>
 #include "extern.h"
 
-static char *
-make_char_printable (int c, int x, int cur_tab_width)
+static const char *
+make_char_printable (char c, int x, int cur_tab_width)
 {
   if (c == '\t')
     return xasprintf ("%*s", cur_tab_width - x % cur_tab_width, "");
-  if (isprint (c))
-    return xasprintf ("%c", c);
   if (c == '\0')
-    return xasprintf ("^@");
+    return "^@";
   else if (c > 0 && c <= '\32')
     return xasprintf ("^%c", 'A' + c - 1);
   else if (c == '\33')
-    return xasprintf ("^[");
+    return "^[";
   else
     return xasprintf ("\\%o", c & 0xff);
 }
@@ -54,17 +52,24 @@ draw_line (size_t line, size_t startcol, Window * wp,
   term_move (line, 0);
 
   /* Draw body of line. */
-  size_t x, i;
+  size_t x, i, line_len = buffer_line_len (get_window_bp (wp), o);
   for (x = 0, i = startcol;; i++)
     {
       term_attrset (highlight && in_region (o, i, r) ? FONT_REVERSE : FONT_NORMAL);
-      if (i >= buffer_line_len (get_window_bp (wp), o) ||
-          x >= get_window_ewidth (wp))
+      if (i >= line_len || x >= get_window_ewidth (wp))
         break;
-      char *s = make_char_printable (get_buffer_char (get_window_bp (wp), o + i),
-                                     x, cur_tab_width);
-      term_addstr (s);
-      x += strlen (s);
+      char c = get_buffer_char (get_window_bp (wp), o + i);
+      if (isprint (c))
+        {
+          term_addch (c);
+          x++;
+        }
+      else
+        {
+          const char *s = make_char_printable (c, x, cur_tab_width);
+          term_addstr (s);
+          x += strlen (s);
+        }
     }
 
   /* Draw end of line. */
@@ -202,15 +207,18 @@ draw_window (size_t topline, Window * wp)
     draw_status_line (topline + get_window_eheight (wp), wp);
 }
 
+static size_t col, cur_topline = 0;
+
 void
 term_redisplay (void)
 {
   /* Calculate the start column if the line at point has to be truncated. */
   Buffer *bp = get_window_bp (cur_wp);
-  size_t col = 0, lastcol = 0, t = tab_width (bp);
+  size_t lastcol = 0, t = tab_width (bp);
   size_t o = window_o (cur_wp);
   size_t lineo = o - get_buffer_line_o (bp);
 
+  col = 0;
   o -= lineo;
   set_window_start_column (cur_wp, 0);
 
@@ -219,7 +227,13 @@ term_redisplay (void)
     {
       col = 0;
       for (size_t p = lp; p < lineo; ++p)
-        col += strlen (make_char_printable (get_buffer_char (bp, o + p), col, t));
+        {
+          char c = get_buffer_char (bp, o + p);
+          if (isprint (c))
+            col++;
+          else
+            col += strlen (make_char_printable (get_buffer_char (bp, o + p), col, t));
+        }
 
       if (col >= ew - 1 || (lp / (ew / 3)) + 2 < lineo / (ew / 3))
         {
@@ -232,7 +246,8 @@ term_redisplay (void)
     }
 
   /* Draw the windows. */
-  size_t cur_topline = 0, topline = 0;
+  cur_topline = 0;
+  size_t topline = 0;
   for (Window *wp = head_wp; wp != NULL; wp = get_window_next (wp))
     {
       if (wp == cur_wp)
@@ -243,7 +258,12 @@ term_redisplay (void)
       topline += get_window_fheight (wp);
     }
 
-  /* Redraw cursor. */
+  term_redraw_cursor ();
+}
+
+void
+term_redraw_cursor (void)
+{
   term_move (cur_topline + get_window_topdelta (cur_wp), col);
 }
 
